/** @file drehgeber.c
 * @par Programm:
 *                 Quellen und Lautstrkesteller
 *
 * @author         Achill Hasler (AHC)
 * @date           05.12.08
 *
 * @par Copyright:
 *                 AH Consulting GmbH
 *
 * @par Compiler:
 *                 avr c fr ATmega48 / 88
 *
 * @brief          Behandlung der Encoder ereigisse.
 *
 * @par History:
 *  <table>
 *    <tr><td>wann</td><td>ver</td><td>wer</td><td>was</td></tr>
 *    <tr><td>26.11.08</td><td>1.0.0</td><td>aha</td><td>
 *    erste Version
 *    </td></tr>
 *    <tr><td>05.12.08</td><td>1.0.1</td><td>aha</td><td>
 *    Fertiggestellt.
 *    </td></tr>
 *  </table>
 */


#include "preamp.h"
#include "timer.h"
#include "rc5.h"

#define DREH_INVALID -128

/* Zhlt die Schritte: von -128..127 */
int8_t volatile volume_ticks = 0;
int8_t volatile channel_steps = 0;

int8_t volatile channel_tick = 0;
uint16_t channel_time = 0;


/* Aufrufen, wenn eine Flanke an einem der Ports festgestellt wurde.
 */
int8_t volume_isr();


/* Mein Drehgeber hat eine Rasterung und bei jedem Raster erzeugt
 * er *vier* Flanken. Man muss also die Anzahl der Schritte etwas
 * mhsam durch ticks teilen.
 */
int8_t channel_isr(int8_t ticks);


static const int8_t drehgeber_trainsitions[] PROGMEM =
{
   0,                       1,           -1, DREH_INVALID,
  -1,                       0, DREH_INVALID,            1,
   1,            DREH_INVALID,            0,           -1,
   DREH_INVALID,           -1,            1,            0
};


/**
 * Interrupt Handler fr Pinchage event auf Port D
 */
#ifdef DOXYGEN
void interrupt_PCINT2_vect(void)
#else
ISR (PCINT2_vect)
#endif
{
  static int8_t last = 0;
  uint8_t input = INPUT_PIN;
  uint8_t pin = last ^ input;
  if (pin & (_BV(ENCA_CA) | _BV(ENCB_CA))) {
    channel_steps += channel_isr(4);
  }
  if (pin & (_BV(ENCA_VO) | _BV(ENCB_VO))) {
    int8_t step = volume_isr();

    if (step != DREH_INVALID) {
      volume_ticks += step;
    }
  }
#ifdef PREAMP_USE_IR
  if (pin & _BV(IR_TSOP)) {
    ir_rc5_pcint_isr();
  }
#endif
  last = input;
}


void drehgeber_enable(void)
{
  const uint8_t pins = _BV(ENCB_CA) | _BV(ENCB_VO) |
                       _BV(ENCA_CA) | _BV(ENCA_VO);
  /* enable pull-up resistors for pins on Port D */
  INPUT_PORT |= pins;

#ifndef PREAMP_USE_IR
  /* enable pin change interupt on Port D */
  PCICR |= _BV(PCIE2); // bei ir
#endif

  /* enable interupt pins on Port D */
  PCMSK2 |= pins;
}


void drehgeber_disable(void)
{
  const uint8_t pins = _BV(ENCB_CA) | _BV(ENCB_VO) |
                       _BV(ENCA_CA) | _BV(ENCA_VO);
  /* disable interupt pins on Port D */
  PCMSK2 &= ~pins;

#ifndef PREAMP_USE_IR /* da ir immer laufen muss */
  /* disable pin change interupt on Port D */
  PCICR &= ~_BV(PCIE2);
#endif

  /* disable pull-up resistors for pins on Port D */
  INPUT_PORT &= ~pins;
}


int8_t volume_isr()
{
  /* Speichert den vorherigen 4-er Zustand in Bits 0 und 1 */
  static uint8_t vol_alt = 0;
  uint8_t vol = vol_alt;

  /* Den aktuellen 4-er Zustand in Bits 2 und 3 einfgen */
  if (INPUT_PIN & _BV(ENCA_VO)) {
    vol |= (1 << 2);
  }
  if (INPUT_PIN & _BV(ENCB_VO)) {
    vol |= (1 << 3);
  }

  /* Den aktuellen Zustand fr den nchsten Aufruf merken */
  vol_alt = vol >> 2;

  /* Zustandswechsel auf Schrittwert abbilden */
  return ((int8_t)pgm_read_byte(&drehgeber_trainsitions[vol]));
}


int8_t channel_isr(int8_t ticks)
{
  /* Speichert den vorherigen 4-er Zustand in Bits 0 und 1 */
  static uint8_t chan_alt = 0;
  uint8_t chan = chan_alt;
  int8_t step, ret_step = 0;

  /* aktuelle Zeit merken */
  set_ticks(&channel_time);
  /* Den aktuellen 4-er Zustand in Bits 2 und 3 einfgen */
  if (INPUT_PIN & _BV(ENCA_CA)) {
    chan |= (1 << 2);
  }
  if (INPUT_PIN & _BV(ENCB_CA)) {
    chan |= (1 << 3);
  }

  /* Den aktuellen Zustand fr den nchsten Aufruf merken */
  chan_alt = chan >> 2;

  /* Zustandswechsel auf Schrittwert abbilden */
  step = (int8_t)pgm_read_byte(&drehgeber_trainsitions[chan]);

  if (step != DREH_INVALID) {
    int8_t t = step + channel_tick;

    /* Wenn |t| >= ticks ist gibt es einen Schritt */
    if (t >= ticks) {
      ret_step = 1;
      /* Schritt aus t entfernen: t -= ticks */
      t -= ticks;
    }
    else if (t <= -ticks) {
      ret_step = -1;
      t += ticks;
    }
    channel_tick = t;
  }

  return ret_step;
}
