/** @file ir-rc5.c
 * @par Programm:
 *                 Quellen und Lautstrkesteller
 *
 * @author         Achill Hasler (AHC)
 * @date           05.12.08
 *
 * @par Copyright:
 *                 AH Consulting GmbH
 *
 * @par Compiler:
 *                 avr c fr ATmega48 / 88
 *
 * @brief          Infrarot RC5 Empfang.
 *
 * @par History:
 *  <table>
 *    <tr><td>wann</td><td>ver</td><td>wer</td><td>was</td></tr>
 *    <tr><td>26.11.08</td><td>1.0.0</td><td>aha</td><td>
 *    erste version fertig
 *    </td></tr>
 *    <tr><td>05.12.08</td><td>1.0.1</td><td>aha</td><td>
 *    verbessert.
 *    </td></tr>
 *  </table>
 */


#include "preamp.h"

#ifdef PREAMP_USE_IR

#include "timer.h"
#include "ir-rc5.h"

/* -----------------------------------------------------------------------------
 * Timing
 * -----------------------------------------------------------------------------
 */
volatile rc5_t ir_data;    /* letztes komplett gelesenes RC5-paket */

#define RC5TIME 	1778.0         /**< in mico sekunden */

/** mindestzeit bis flanke wechselt */
#define RC5_TICKS_MIN	(uint8_t)(US_TO_TICKS(RC5TIME * 0.4) + 0.5)

/** ab dieser zeit kann es sich nur um einen langen flankenwechsel handeln */
#define RC5_TICKS_1_2	(uint8_t)(US_TO_TICKS(RC5TIME * 0.8) + 0.5)

/** wenn flankenwechsel lnger dauert dann ungltig */
#define RC5_TICKS_MAX	(uint8_t)(US_TO_TICKS(RC5TIME * 1.2) + 0.5)


/*
 * Interrupt Serviceroutine,
 * muss nach pinchange aufgerufen werden
 */
void ir_rc5_pcint_isr(void)
{
  static uint8_t bit_time;      /* verstrichene  Zeit seit letzter Flanke */
  static uint16_t rc5_tmp = 0;  /* bis anhin gelesene rc5 bits */
  uint16_t tmp = rc5_tmp;				/* arbeite mit Register (schneller) */
  uint8_t now = tick_count.u8;

  uint8_t diff_time = (now > bit_time)
                    ? now - bit_time
                    : 0xff - bit_time + now + 1;

  if(diff_time < RC5_TICKS_MIN) { /* zu kurz */
    tmp = 0;
    /* bittime zurckstellen */
    bit_time = now; 
  }
  if(diff_time > RC5_TICKS_MAX) { /* zu lang */
    tmp = 0;
  }

  /* aktueller portstatus. active low */
  uint8_t rc5_bit = (INPUT_PIN & _BV(IR_TSOP)) ? 0 : 1;

  /* start or long pulse time */
  if ( ((tmp == 0) && rc5_bit) || 
    ((diff_time > RC5_TICKS_1_2) && (diff_time < RC5_TICKS_MAX)) ) {
    if (tmp < 0x2000) { /* startbit noch nicht angekommen */
      tmp <<= 1;        /* shift */
      tmp |= rc5_bit;   /* insert new bit */
    }
    if (tmp & 0x2000) { /* rc5 komplett empfangen */
      /* nur wenn 14 bits empfangen sind */
      ir_data.value = tmp;
      tmp = 0;
    }
    bit_time = now;     /* bittime zurckstellen */
  }

  rc5_tmp = tmp;
}


/*
 * IR-Daten lesen
 * return Wert von ir_data, loescht anschliessend ir_data
 */
rc5_t ir_rc5_read(void)
{
  rc5_t retvalue = ir_data;
  ir_data.value = 0;
  return retvalue;
}

#endif /* PREAMP_USE_IR */
