/** @file ir-rc5.h
 * @par Programm:
 *                 Quellen und Lautstrkesteller
 *
 * @author         Achill Hasler (AHC)
 * @date           05.12.08
 *
 * @par Copyright:
 *                 AH Consulting GmbH
 *
 * @par Compiler:
 *                 avr c fr ATmega48 / 88
 *
 * @brief          Inlude fr Infrarot RC5 Empfang.
 *
 * Infos ber RC6: http://www.sbprojects.com/knowledge/ir/ir.htm
 *
 * @par History:
 *  <table>
 *    <tr><td>wann</td><td>ver</td><td>wer</td><td>was</td></tr>
 *    <tr><td>26.11.08</td><td>1.0.0</td><td>aha</td><td>
 *    erste version fertig
 *    </td></tr>
 *    <tr><td>05.12.08</td><td>1.0.1</td><td>aha</td><td>
 *    verbessert.
 *    </td></tr>
 *  </table>
 */

#ifndef PREAMP_IR_RC5_H_
#define PREAMP_IR_RC5_H_

#ifdef PREAMP_USE_IR


/**
 * Datentyp fr ein rc5 Kommando
 */
typedef union {
  struct {
    uint8_t command : 6;
    uint8_t system : 5;
    uint8_t toggle : 1;
    uint8_t start_opt : 1;
    uint8_t start : 1;
    uint8_t filler : 2;
  } bits;
  uint16_t value;
} rc5_t;


/**< letztes komplett gelesenes RC5-Paket */
extern volatile rc5_t ir_data;


/**
 * Interrups von ir zulassen
 */
inline void ir_rc5_enable(void)
{
  PCMSK2 |= _BV(IR_TSOP); /* enable ir interrupts */
}


/**
 * Init IR-System
 */
inline void ir_rc5_init(void)
{
  INPUT_DDR  &= ~IR_TSOP; 	/* Pin auf Input */

  /* enable pin change interupt on Port D */
  PCICR |= _BV(PCIE2);
  ir_rc5_enable();
}


/**
 * Interrups von ir abschalten
 */
inline void ir_rc5_disable(void)
{
  PCMSK2 &= ~_BV(IR_TSOP); /* disable ir interrupt */
}


/**
 * hat es ein neues IR-Paket?
 * @return 1 wenn daten anstehen, sonst 0
 */
inline uint8_t ir_rc5_has_data(void) {
  return ir_data.bits.start;
}


/**
 * IR-Daten lesen
 * @return Wert von ir_data, loescht anschliessend ir_data
 */
rc5_t ir_rc5_read(void);


/**
 * Interrupt Serviceroutine fr IR-Empfaenger,
 * muss nach pinchange aufgerufen werden
 */
void ir_rc5_pcint_isr(void);

#endif /* PREAMP_USE_IR */

#endif /* PREAMP_IR_RC5_H_ */
