/** @file preamp.c
 * @par Programm:
 *                 Quellen und Lautstrkesteller
 *
 * @author         Achill Hasler (AHC)
 * @date           05.12.08
 *
 * @par Copyright:
 *                 AH Consulting GmbH
 *
 * @par Compiler:
 *                 avr c fr ATmega48 / 88
 *
 * @brief          Hauptfunktionen.
 *
 * @par History:
 *  <table>
 *    <tr><td>wann</td><td>ver</td><td>wer</td><td>was</td></tr>
 *    <tr><td>26.11.08</td><td>1.0.0</td><td>aha</td><td>
 *    erste version fertig
 *    </td></tr>
 *    <tr><td>05.12.08</td><td>1.0.1</td><td>aha</td><td>
 *    code verbessert.
 *    </td></tr>
 *  </table>
 */

#include "preamp.h"
#include "relais.h"
#include "timer.h"
#include "drehgeber.h"
#include "eeprom.h"

volatile amp_state state = off;

amp_inputs chan;  /* das ist der wunsch kanal */
amp_volume vol;   /* und die wunsch lautstrke */

int8_t channel_vol_offset[5];

/* ------- locale functions ------- */
uint8_t get_actual_channel(void);



/* ------- code ------- */
void mute_on(void)
{
  if (state == on) {
    state = mute;
    vol.bits.mute = 0;
  }
}


void mute_off(void)
{
  if (state == mute) {
    state = on;
    vol.bits.mute = 1;
  }
}


void set_channel(uint8_t input)
{
  if (chan.bits.input != input) {
    /* kanal hat gewechselt */
    uint8_t act = get_actual_channel();

    /* offset des alten input kanals */
    int8_t offset_old = channel_vol_offset[act];

    chan.bits.input = input;
    act = get_actual_channel();
    /* offset des neuen input kanals */
    int8_t offset_new = channel_vol_offset[act];

    int8_t volume_diff = offset_new - offset_old;
    /* volume anhand der differenz korrigieren */
    if (volume_diff > 0) {
      increment_volume(volume_diff);
    }
    if (volume_diff < 0) {
      decrement_volume(-volume_diff);
    }
  }
}


void increment_channel(void)
{
  uint8_t input = chan.bits.input;
  if (input < _BV(PC4)) {
    set_channel(input << 1);
  }
}


void decrement_channel(void)
{
  uint8_t input = chan.bits.input;
  if (input > _BV(PC0)) {
    set_channel(input >> 1);
  }
}


/**
 * Anhand der durchgangszeit des volume encoder wird der nchste
 * Lautstrken offset bestimmt.
 * @param start_time Zeit bei der das este von 4 ticks erreicht wurde
 */
int8_t get_volume_step(uint16_t start_time)
{
  uint16_t diff = tick_count.u16 - start_time;

  if (diff < MS_TO_TICKS(3)) {
    return 0; /* bin am prellen */
  }
  else if (diff < MS_TO_TICKS(7)) {
    return 8;
  }
  else if (diff < MS_TO_TICKS(20)) {
    return 4;
  }
  /* MS_TO_TICKS(ms)		(((ms) * 1000UL) / TIMER_STEPS)
                           30 * 1000 / 100 = 300 Ticks
   */
  else if (diff < MS_TO_TICKS(30)) {
    return 2;
  }
  return 1;
}


void increment_volume(int8_t step)
{
  if (vol.bits.volume + step < _BV(7)) {
    vol.bits.volume += step;
  }
  else {
    vol.bits.volume = _BV(7) - 1;
  }
}


void decrement_volume(int8_t step)
{
  if (vol.bits.volume >= step) {
    vol.bits.volume -= step;
  }
  else {
    vol.bits.volume = 0;
  }
}


uint8_t get_actual_channel(void)
{
  uint8_t act_channel = 4;
  uint8_t input = chan.bits.input;

  if (input == _BV(PC0)) {
    act_channel = 0;
  }
  else if (input == _BV(PC1)) {
    act_channel = 1;
  }
  else if (input == _BV(PC2)) {
    act_channel = 2;
  }
  else if (input == _BV(PC3)) {
    act_channel = 3;
  }
  return (act_channel);
}


void reset_channel_volume(void)
{
  uint8_t act = get_actual_channel();
  int8_t offset = -channel_vol_offset[act];

  if (offset > 0) {
    increment_volume(offset);
  }
  if (offset < 0) {
    decrement_volume(-offset);
  }
  for (uint8_t i = 0; i < 5; i++) {
    channel_vol_offset[i] = 0;
  }
}


void increment_channel_volume(int8_t step)
{
  uint8_t act = get_actual_channel();
  int8_t offset_volume = channel_vol_offset[act];

  if (offset_volume + step <= CHANNEL_MAX_OFFSET) {
    offset_volume += step;
  }
  else {
    offset_volume = CHANNEL_MAX_OFFSET;
  }
  channel_vol_offset[act] = offset_volume;
  increment_volume(step);
}


void decrement_channel_volume(int8_t step)
{
  uint8_t act = get_actual_channel();
  int8_t offset_volume = channel_vol_offset[act];

  if (offset_volume - step >= -CHANNEL_MAX_OFFSET) {
    offset_volume -= step;
  }
  else {
    offset_volume = -CHANNEL_MAX_OFFSET;
  }
  channel_vol_offset[act] = offset_volume;
  decrement_volume(step);
}


void test_channel(void)
{
  if (channel_steps != 0) {
    if (channel_steps > 0) {
      increment_channel();
    }
    else {
      decrement_channel();
    }
    channel_steps = 0;
  }
  if (after_ms(&channel_time, 1500)) {
    /* wird nach 1.5 s nichts tun ausgefuehrt,
      null durchgang wird neu justiert */
    channel_time = 0;
    channel_tick = 0;
  }
}


void test_volume(int8_t ticks)
{
  static uint16_t time = 0;
  static int8_t dir = 0;
  /* lautstaerke nur verstellen wenn aktiv */
  if (state != on && state != cancel_mute) return;

//  if (after_ms(&time, 3000)) { // kein Platz, unntig
    /* wird nach 3 s nichts tun ausgefuehrt,
      null durchgang wird neu justiert */
//    volume_ticks = 0;
//    dir = 0;
//    time = 0;
//  }
  if (volume_ticks != 0) {
    if (dir == 0) {
      if (volume_ticks > 0) {
        dir = 1;
      }
      else {
        dir = -1;
      }
      set_ticks(&time);
    }

    /* Wenn |volume_steps| >= ticks ist gibt es einen schritt */
    int8_t stepw = 1;
    if (volume_ticks >= ticks) {
      if (dir == 1) {
        stepw *= get_volume_step(time);
      }
      /* wenn mute gedrckt, dann kanal offset verstellen */
      uint8_t volume_button = INPUT_PIN & _BV(BUTT_VO);
      if (volume_button == 0) { /* mute taste gedrckt */
        increment_channel_volume(stepw);
        state = cancel_mute;    /* beim loslassen der taste mute verhindern */
      }
      else {
        increment_volume(stepw);
      }

      /* Schritt aus volume_ticks entfernen */
      volume_ticks -= ticks;
      dir = 0;
    }
    else if (volume_ticks <= -ticks) {
      if (dir == -1) {
        stepw *= get_volume_step(time);
      }
      /* wenn mute gedrckt, dann kanal offset verstellen */
      uint8_t volume_button = INPUT_PIN & _BV(BUTT_VO);
      if (volume_button == 0) { /* mute taste gedrckt */
        decrement_channel_volume(stepw);
        state = cancel_mute;    /* beim loslassen der taste mute verhindern */
      }
      else {
        decrement_volume(stepw);
      }

      volume_ticks += ticks;
      dir = 0;
    }
  }
}
