/** @file rc5.c
 * @par Programm:
 *                 Quellen und Lautstrkesteller
 *
 * @author         Achill Hasler (AHC)
 * @date           05.12.08
 *
 * @par Copyright:
 *                 AH Consulting GmbH
 *
 * @par Compiler:
 *                 avr c fr ATmega48 / 88
 *
 * @brief          Infrarot RC5 Befehle.
 *
 * @par History:
 *  <table>
 *    <tr><td>wann</td><td>ver</td><td>wer</td><td>was</td></tr>
 *    <tr><td>26.11.08</td><td>1.0.0</td><td>aha</td><td>
 *    erste version fertig
 *    </td></tr>
 *    <tr><td>05.12.08</td><td>1.0.1</td><td>aha</td><td>
 *    verbessert.
 *    </td></tr>
 *  </table>
 */

#include "preamp.h"

#ifdef PREAMP_USE_IR

#include "timer.h"
#include "rc5.h"
#include "relais.h"

/**
 * Abstand im ms von zwei rc5 Komandos
 */
const uint8_t ir_repeat_ms = 110;

/**
 * Empfangenes rc5 Komando wiederholt verarbeiten
 * @param command rc5 Befehl
 */
inline void rc5_repeated_event(uint8_t command);

/**
 * Empfangenes rc5 Komando verarbeiten
 * liefert true, wenn Komando verarbeitet wurde
 * @param command rc5 Befehl
 * @return 1 wenn Komando augefhrt, 0 wenn unbekannt
 */
inline uint8_t rc5_event(uint8_t command);

#define EQ1(comm, par1) ((comm) == (par1))
#define EQ2(comm, par1, par2) ((comm) == (par1) || (comm) == (par2))
#define EQ3(comm, par1, par2, par3) ((comm) == (par1) || (comm) == (par2) || (comm) == (par3))

#define REPEAD_AFTER_COMMAND 4

/**
 * Diese Werte werden mittels programm ln_list.c erzeugt.
 * Sie beschreiben die Zeit in ms fr den nchten Volumenwechsel bei der
 * aktuellen Lautstrke.
 * Leider kann ich nicht alle werte verwenden, da speicherplatzproblem.
 */

/* ln liste von 5 .. 255 mit 128 stufen bei schrittweite 2. */
static const uint8_t volume_steps[] PROGMEM = {
/*0*/ 5,
/*2*/ 5,
/*4*/ 5,
/*6*/ 6,
/*8*/ 6,
/*10*/ 6,
/*12*/ 7,
/*14*/ 7,
/*16*/ 8,
/*18*/ 8,
/*20*/ 9,
/*22*/ 9,
/*24*/ 10,
/*26*/ 11,
/*28*/ 11,
/*30*/ 12,
/*32*/ 13,
/*34*/ 14,
/*36*/ 15,
/*38*/ 16,
/*40*/ 17,
/*42*/ 18,
/*44*/ 19,
/*46*/ 20,
/*48*/ 22,
/*50*/ 23,
/*52*/ 25,
/*54*/ 26,
/*56*/ 28,
/*58*/ 30,
/*60*/ 32,
/*62*/ 34,
/*64*/ 36,
/*66*/ 38,
/*68*/ 41,
/*70*/ 43,
/*72*/ 46,
/*74*/ 49,
/*76*/ 52,
/*78*/ 55,
/*80*/ 59,
/*82*/ 63,
/*84*/ 67,
/*86*/ 71,
/*88*/ 76,
/*90*/ 81,
/*92*/ 86,
/*94*/ 91,
/*96*/ 97,
/*98*/ 103,
/*100*/ 110,
/*102*/ 117,
/*104*/ 125,
/*106*/ 133,
/*108*/ 141,
/*110*/ 150,
/*112*/ 160,
/*114*/ 170,
/*116*/ 181,
/*118*/ 192,
/*120*/ 205,
/*122*/ 218,
/*124*/ 232,
/*126*/ 247,
};


uint8_t get_next_step(void)
{
  uint8_t pos;
//  pos = (vol.bits.volume); /* programmspeicher berlauf! */
  pos = VOLUME_PORT & VOLUME_MASK;
  pos /= 2; /* schrittweite */
  return pgm_read_byte(&volume_steps[pos]);
}

void test_ir(void)
{
  static uint16_t last_command_time = 1;
  static uint8_t last_command;
  static uint8_t next_step_ms = 0;

  static uint8_t repeat_count = 0;
  static uint16_t repeat_count_time = 0;

  if (ir_rc5_has_data()) {
    rc5_t data = ir_rc5_read();

    if (EQ2(data.bits.system, RC5_AMP1, RC5_TUNER)) {
      /* ist fr mich. meine universal FB liefert leider fr keinen systemcode
         alle funktionen. nun weden eben zwei gerteklassen untersttzt */
      uint8_t command = data.bits.command;
      if (!data.bits.start_opt) {  /* wenn 2'tes startbit neg, dann erweitertes kommando */
        command += 64; // 64 .. 127
      }

      if (after_ms(&last_command_time, ir_repeat_ms + 20)) {
        /* taste wurde zum ersten mal gedrckt */
        last_command = command;
        if (rc5_event(command)) {
          set_ticks(&last_command_time);
          set_ticks(&repeat_count_time);
          repeat_count = 0;
          next_step_ms = get_next_step();
        }
      }
      else {
        /* bin am repitieren */
        if (repeat_count < REPEAD_AFTER_COMMAND) { repeat_count++; }
        /* ab der n repitition, volume logarithmisch verndern
           da sonst keine 1 stufigen komandos mittels tippen gegeben werden
           knnen. (ca. 0.3 s) */
        set_ticks(&last_command_time);
      }
    }
  }

  if (state >= on) {
    if (repeat_count == REPEAD_AFTER_COMMAND) {
      /* nach 130 ms keine taste -> wird gestoppt */
      if (after_ms(&last_command_time, ir_repeat_ms + 20)) {
        repeat_count_time = 0;
        repeat_count = 0;
      }
      /* solange rc5 signal wird volume logarithmisch verndert */
      if (after_ms(&repeat_count_time, next_step_ms)) {
        rc5_repeated_event(last_command);
        set_ticks(&repeat_count_time);
        next_step_ms = get_next_step();
      }
    }
  }
}



inline void rc5_repeated_event(uint8_t command)
{
  if (EQ1(command, RC5_VOLUME_UP)) {
    increment_volume(1);
  }
  else if (EQ1(command, RC5_VOLUME_DOWN)) {
    decrement_volume(1);
  }
}


inline uint8_t rc5_event(uint8_t command)
{
  uint8_t valid_command = 1;

  if (EQ2(command, RC5_POWER, RC5_STANDBY)) {
    if (state == off) {
      switch_on();
    }
    else {
      switch_off();
    }
  }
  else if (EQ1(command, RC5_VOLUME_UP)) {
    increment_volume(1);
  }
  else if (EQ1(command, RC5_VOLUME_DOWN)) {
    decrement_volume(1);
  }
  else if (EQ1(command, RC5_MUTE)) {
    if (state == on) {
      mute_on();
    }
    else if (EQ1(state, mute)) {
      mute_off();
    }
  }
  else if ((command >= RC5_1) && (command <= RC5_5)) {
    set_channel(_BV(command - RC5_1));// Kanal 1 ist Bit 0
  }
//  else if (EQ3(command, RC5_CANNEL_UP, RC5_INDEX_UP, RC5_VORWARD)) {
  else if (EQ1(command, RC5_CANNEL_UP)) {
    increment_channel();
  }
//  else if (EQ3(command, RC5_CANNEL_DOWN, RC5_INDEX_DOWN, RC5_REWIND)) {
  else if (EQ1(command, RC5_CANNEL_DOWN)) {
    decrement_channel();
  }
  else if (EQ1(command, PREAMP_RC5_CHANNEL_VOLUME_UP)) {
    increment_channel_volume(1);
  }
  else if (EQ1(command, PREAMP_RC5_CHANNEL_VOLUME_DOWN)) {
    decrement_channel_volume(1);
  }
//  else if (EQ1(command, PREAMP_RC5_RESET)) {
//    reset_channel_volume();
//  }
  else {
    valid_command = 0;
  }

  return (valid_command);
}

#endif /* PREAMP_USE_IR */
