/** @file relais.c
 * @par Programm:
 *                 Quellen und Lautstrkesteller
 *
 * @author         Achill Hasler (AHC)
 * @date           05.12.08
 *
 * @par Copyright:
 *                 AH Consulting GmbH
 *
 * @par Compiler:
 *                 avr c fr ATmega48 / 88
 *
 * @brief          Relais steuerung.
 *
 * @par History:
 *  <table>
 *    <tr><td>wann</td><td>ver</td><td>wer</td><td>was</td></tr>
 *    <tr><td>26.11.08</td><td>1.0.0</td><td>aha</td><td>
 *    erste version fertig
 *    </td></tr>
 *    <tr><td>05.12.08</td><td>1.0.1</td><td>aha</td><td>
 *    mute verbessert.
 *    </td></tr>
 *  </table>
 */

#include "preamp.h"
#include "timer.h"
#include "relais.h"
#include "drehgeber.h"
#include "eeprom.h"
#include "ir-rc5.h"

amp_relais_state chan_relais_state;
amp_relais_state vol_relais_state;

uint16_t power_relais_time = 0;

/* t_save = (t_off - t_on) * 2 = 1 ms -> (3.5 - 3) * 2
 */

const uint8_t t_on = 3;
const uint8_t t_save = 1;


void adjust_channel(void)
{
  static amp_inputs actual_chan;  /* das steht im register */

  static uint16_t time = 0;
  static uint8_t next_stable = 0;

  if (chan_relais_state == rel_stable && actual_chan.value != chan.value) {
    set_ticks(&time);
    chan_relais_state = rel_switching_off;
    next_stable = chan.value;
    /* setze nur gemeinsame bits */
    CHANNEL_PORT &= (next_stable | ~CHANNEL_MASK);
    return;
  }
  if (chan_relais_state == rel_switching_on && after_ms(&time, t_on)) {
 	  /* nach t_on ms sind die relais gesetzt */
    chan_relais_state = rel_stable;
    actual_chan.value = next_stable;
    time = 0;
    return;
  }
  if (chan_relais_state == rel_switching_off && after_ms(&time, t_save)) {
 	  /* nach t_save ms werden auch die restlichen relais gesetzt */
    chan_relais_state = rel_switching_on;
    /* setze nun alle bits */
    CHANNEL_PORT |= next_stable;
    return;
  }
  if (chan_relais_state == rel_undefined) {
    set_ticks(&time);
    chan_relais_state = rel_switching_on;
    next_stable = chan.value;
    CHANNEL_PORT |= next_stable;
  }
}


void adjust_volume(void)
{
  static amp_volume actual_vol;

  static uint16_t time = 0;
  static uint8_t next_stable = 0;

  uint8_t whish = vol.value;
  if (vol.bits.mute == 0) { /* mute wird verlangt */
    uint8_t data;
    data = JUMPER_PIN;
    if (data & _BV(JUMPER)) {
      /* jumper ist offen, active low */
      if (vol_relais_state == rel_stable && actual_vol.bits.volume > 0) {
        /* eine stufe leiser drehen, kein mute relais setzen */
        whish = (actual_vol.bits.volume - 1) | _BV(MUTE);
      }
      else {
        whish = _BV(MUTE);
      }
    }
    else {
      whish = 0; /* ruhe in der kiste. auch mute relais aktiv */
    }
  }

  if (vol_relais_state == rel_stable && actual_vol.value != whish) {
    set_ticks(&time);
    vol_relais_state = rel_switching_off;
    next_stable = whish;
    /* setze nur gemeinsame bits */
    VOLUME_PORT = actual_vol.value & next_stable;
    return;
  }
  if (vol_relais_state == rel_switching_on && after_ms(&time, t_on)) {
 	  /* nach t_on ms sind die relais gesetzt */
    vol_relais_state = rel_stable;
    actual_vol.value = next_stable;
    time = 0;
    return;
  }
  if (vol_relais_state == rel_switching_off && after_ms(&time, t_save)) {
 	  /* nach t_save ms werden auch die restlichen relais gesetzt */
    vol_relais_state = rel_switching_on;
    VOLUME_PORT = next_stable; /* setze nun alle bits */
    return;
  }
  if (vol_relais_state == rel_undefined) {
    set_ticks(&time);
    vol_relais_state = rel_switching_on;
    next_stable = whish;
    VOLUME_PORT = next_stable;
  }
}


void input_relais_reset(void)
{
  chan_relais_state = rel_undefined;
  vol_relais_state = rel_undefined;
}


void switch_on(void)
{
  set_ticks(&power_relais_time);
  VOLUME_PORT = _BV(MUTE); /* turn Volume OFF mute nicht aktiv */
  input_relais_reset();
  power_relais_on();
  state = switching_on;
}


void switch_off(void)
{
  write_defaults();
  set_ticks(&power_relais_time);
  VOLUME_PORT = _BV(MUTE); /* turn Volume OFF mute nicht aktiv */
  power_relais_off();
  drehgeber_disable();
  state = switching_off;
}


void main_relais_swiched(void)
{
  if (after_ms(&power_relais_time, MAIN_WAITING_TIME)) {
    /* wird 2 s nach dem power_on / power_off ausgefuehrt
      relais und trafo sind dann bereit */
    if (state == switching_on) {
      read_defaults();
      drehgeber_enable();
      state = on;

      /* ist mute immer noch gedrckt? */
      uint8_t volume_button = INPUT_PIN & _BV(BUTT_VO);

      /* taste noch gedrckt und verstrker betriebsbereit */
      if (volume_button == 0) {
        state = cancel_mute; /* mute verhindern beim loslassen */ 
      }
    }
    /*  Relais und Trafo sind jetzt aus */
    else if (state == switching_off) {
      CHANNEL_PORT &= ~CHANNEL_MASK; /* turn all Channels LED OFF */
      state = off;
    }
    power_relais_time = 0;
#ifdef PREAMP_USE_IR
    ir_rc5_read(); /* ir puffer leeren */
#endif
  }
}
