/** @file timer.h
 * @par Programm:
 *                 Quellen und Lautstrkesteller
 *
 * @author         Achill Hasler (AHC)
 * @date           05.12.08
 *
 * @par Copyright:
 *                 AH Consulting GmbH
 *
 * @par Compiler:
 *                 avr c fr ATmega48 / 88
 *
 * @brief          Inlude fr Zeitfunktionen.
 *
 * @par History:
 *  <table>
 *    <tr><td>wann</td><td>ver</td><td>wer</td><td>was</td></tr>
 *    <tr><td>26.11.08</td><td>1.0.0</td><td>aha</td><td>
 *    erste version fertig
 *    </td></tr>
 *    <tr><td>02.12.08</td><td>1.0.1</td><td>aha</td><td>
 *    automatische clockeinstellung.
 *    </td></tr>
 *  </table>
 */

#ifndef PREAMP_TIMER_H_
#define PREAMP_TIMER_H_


/**
 * Mikrosekunden, die zwischen zwei timer-aufrufen liegen
 */
#define TIMER_STEPS 100UL


/**
 * Makro zur Umrechnung von us in Ticks
 * (ms / ticks evtl. nach uint32_t casten, fr grosse werte)
 */
#define US_TO_TICKS(us)		((us) / TIMER_STEPS)


/**
 * Makro zur umrechnung von ticks in ms
 * (ms / ticks evtl. nach uint32_t casten, fr grosse werte)
 */
#define TICKS_TO_MS(ticks)	(((ticks) * TIMER_STEPS) / 1000UL)


/**
 * Makro zur umrechnung von ms in ticks
 * (ms / ticks evtl. nach uint32_t casten, fr grosse Werte)
 */
#define MS_TO_TICKS(ms)		(((ms) * 1000UL) / TIMER_STEPS)


/**
 * Union fr tick_count in 8, 16 und 32 Bit
 */
typedef union {
  uint32_t u32;   /**< 32 bit integer */
  uint16_t u16;   /**< 16 bit integer */
  uint8_t u8;     /**< 8 bit integer */
} tick_count_t;


/**
 * ein tick alle 100 us
 */
extern volatile tick_count_t tick_count;


/**
 * Setzt die Systemzeit zurck auf 0
 */
static inline void timer_reset(void) {
  tick_count.u32 = 0;
}



/**
 * Teilereinstellung der CPU
 *
 * Mgliche Teiler:
 *               CS20 -> 1\n
 *        CS21        -> 8\n
 *        CS21 & CS20 -> 32\n
 * CS22               -> 64\n
 * CS22 &        CS20 -> 128\n
 * CS22 & CS21        -> 256\n
 * CS22 & CS21 & CS20 -> 1024
 *
 * Dabei darf folgender Wert nicht grsser als 255 werden
 *
 *  ((((F_CPU / 1000000) * TIMER_STEPS) / TIMER_TEILER) - 1)
 *
 *  F_CPU = 8000000, TIMER_STEPS = 100, TIMER_TEILER = 8 -> 99 ok!
 */
#define TIMER_TEILER 8


#if TIMER_TEILER == 1
#define TIMER_PRESCALER                         _BV(CS20)

#elif TIMER_TEILER == 8
#define TIMER_PRESCALER             _BV(CS21)

#elif TIMER_TEILER == 32
#define TIMER_PRESCALER             _BV(CS21) | _BV(CS20)

#elif TIMER_TEILER == 64
#define TIMER_PRESCALER _BV(CS22)

#elif TIMER_TEILER == 128
#define TIMER_PRESCALER _BV(CS22)             | _BV(CS20)

#elif TIMER_TEILER == 256
#define TIMER_PRESCALER _BV(CS22) | _BV(CS21)

#elif TIMER_TEILER == 1024
#define TIMER_PRESCALER _BV(CS22) | _BV(CS21) | _BV(CS20)
#else
#error TIMER_TEILER korrekt setzen!
#endif


/**
 * Frequenz von Timer 2 in Hz
 */
#define TIMER_CLOCK	(1000000UL/TIMER_STEPS)

/**
 * Stellt den tick_counter
 */
void set_ticks(uint16_t* ticks);


/**
 * Nicht blockierende Wartefunktion
 *
 * Falls die funktion mit old_ticks = 0 aufgerufen wird, liefert sie immer 0.
 *
 * Die maximale Wartezeit im ms bertrgt: (da uint16_t)
 * 0xffff * TIMER_STEPS / 1000
 * bei TIMER_STEPS = 100
 * kann dabei maximal 4095 millisekunden gewartet werden.
 * @param old_ticks Adresse einer variablen die mit set_ticks gestellt wurde
 * @param ms Wartezeit in milisekunden
 * @return 1 wenn zeit abgelaufen, 0 sonst
 */
uint8_t after_ms(uint16_t* old_ticks, uint16_t ms);


/**
 * Initialisiert timer 2, und startet ihn, dabei wird auch
 * die uhr gestellt.
 */
inline void timer_init(void)
{
  /* auf meinem SK500 ATmega48 Vss 5V 8Mhz interner clock, luft die eingestellte
     oszilator kalibritation von 167 zu schnell. (1 sec in 22)
     dammit wird das ding gedrosselt */

  OSCCAL = 158;  /* immer noch etwas zu schnell, aber 157 ist zu langsam */

  TCNT2  = 0x00;            /* TIMER vorladen */

#if defined(__AVR_ATmega88__) || defined(__AVR_ATmega48__)
  TCCR2A = _BV(WGM21);	/* CTC Mode */
  TCCR2B = TIMER_PRESCALER;		/* Prescaler = CLK/8 */
  OCR2A = ((((F_CPU / 1000000) * TIMER_STEPS) / TIMER_TEILER) - 1);
  TIMSK2  |= _BV(OCIE2A);	/* TIMER2 Output Compare Match A Interrupt an */
#else
  /* use CLK/64 prescale value, clear timer/counter on compare match */
  TCCR2 = _BV(WGM21) | TIMER_PRESCALER;
  OCR2 = ((((F_CPU / 1000000) * TIMER_STEPS) / TIMER_TEILER) - 1);
  TIMSK  |= _BV(OCIE2);	/* enable Output Compare 0 overflow interrupt */
#endif
}

#endif /* PREAMP_TIMER_H_ */
